import os
import logging
import threading
import tempfile
from typing import Tuple
try:
    from spython.main import get_client
    from spython.utils import check_install
    USE_DOCKER = False
    client = get_client(quiet=True)
    if not check_install():
        raise ImportError("Singularity not available")
except ImportError:
    from docker import from_env
    USE_DOCKER = True
    client = from_env()

logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)

# Create a temporary directory to hold all the writable parts of the configuration + any
# data stored in the databases. We put this in a special user directory so that all the files
# are self-conained
tmpdir = os.makedirs(os.getcwd() + "/scratch/tmp/", exist_ok=True)
# make sure this exists in the 'scratch' directory
dbdir = tempfile.TemporaryDirectory(prefix=os.getcwd() + "/scratch/tmp/")

class Service:
    singularity_container: str
    docker_container: Tuple[str, str]

    @classmethod
    def pull(cls, singularity_filename: str):
        if USE_DOCKER:
            logging.info(f"Pulling contaner: {cls.docker_container}")
            client.images.pull(cls.docker_container[0], tag=cls.docker_container[1])
        elif not os.path.exists(singularity_filename):
            logging.info(f"Pulling contaner: {cls.singularity_container}")
            client.pull(image=cls.singularity_container)
        else:
            logging.info(f"Using existing image: {singularity_filename}")

    @classmethod
    def run(cls):
        raise NotImplementedError

    @classmethod
    def stop(cls, instance):
        client.instance.instance.stop(f"instance://{instance.name}")


class MongoDB(Service):
    singularity_container = "docker://mongo:latest"
    docker_container = ("mongo", "latest")

    @classmethod
    def run(cls, image_file: str = "/sw/apps/singularity-images/mines_data_engineering/mongo.sif"):
        cls.pull(image_file)
        if USE_DOCKER:
            return client.containers.run(cls.docker_container[0], detach=True, auto_remove=False, name="data-engineering-mongodb")
        else:
            inst = client.instance.instance(image=image_file, options=[f"-B {dbdir.name}:/data/db", f"-B {dbdir.name}:/tmp"])
            def _run_mongo():
                client.execute(image=f"instance://{inst.name}", command=["mongod"])
            t = threading.Thread(target=_run_mongo)
            t.start()
            return inst


class TimescaleDB(Service):
    singularity_container = "docker://timescale/timescaledb:latest-pg14"
    docker_container = ("timescale/timescaledb", "latest-pg14")

    @classmethod
    def run(cls, image_file: str = "/sw/apps/singularity-images/mines_data_engineering/timescaledb-pg14.sif"):
        cls.pull(image_file)
        if USE_DOCKER:
            return client.containers.run(cls.docker_container[0], detach=True, auto_remove=False, name="data-engineering-timescaledb")
        else:
            os.environ["POSTGRES_PASSWORD"] = "password"
            inst = client.instance.instance(image=image_file, options=[f"-B {dbdir.name}:/var/lib/postgresql/data", f"-B {dbdir.name}:/var/run/postgresql", f"--env=POSTGRES_PASSWORD"])
            def _run_timescale():
                client.execute(image=f"instance://{inst.name}", command=["/usr/local/bin/docker-entrypoint.sh", "postgres"])
            t = threading.Thread(target=_run_timescale)
            t.start()
            return inst



import logging
import time
from mines_data_engineering.container import MongoDB, TimescaleDB, dbdir

logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)

_mongo_instance = None
_pg_instance = None

def start_mongo(image_file: str =  "/sw/apps/singularity-images/mines_data_engineering/mongo.sif"):
    """
    Starts MongoDB and returns the connection string
    """
    global _mongo_instance
    if _mongo_instance is not None:
        logging.info("MongoDB is already running!")
        return "mongodb://" + f"{dbdir.name}/mongodb-27017.sock".replace('/', '%2F')

    logging.info("Starting MongoDB")
    _mongo_instance = MongoDB.run(image_file)
    logging.info("Sleeping for 2 seconds to let MongoDB start")
    time.sleep(2)
    return "mongodb://" + f"{dbdir.name}/mongodb-27017.sock".replace('/', '%2F')


def stop_mongo():
    global _mongo_instance
    MongoDB.stop(_mongo_instance)
    _mongo_instance = None


def start_postgres(image_file: str =  "/sw/apps/singularity-images/mines_data_engineering/timescaledb-pg14.sif"):
    """
    Starts Postgres w/ TimescaleDB extension and returns the connection string
    """
    global _pg_instance
    if _pg_instance is not None:
        logging.info("Postgres is already running!")
        return f"user=postgres password=password host={dbdir.name}"
    logging.info("Starting Postgres")
    _pg_instance = TimescaleDB.run(image_file)
    logging.info("Sleeping for 20 seconds to let Postgres start")
    time.sleep(20)
    return f"user=postgres password=password host={dbdir.name}"


def stop_postgres():
    global _pg_instance
    TimescaleDB.stop(_pg_instance)
    _pg_instance = None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mines_data_engineering']

package_data = \
{'': ['*']}

install_requires = \
['docker>=6.0.1,<7.0.0',
 'psycopg2-binary>=2.9.5,<3.0.0',
 'pymongo>=4.3.3,<5.0.0',
 'spython>=0.3.0,<0.4.0',
 'xattr>=0.10.1,<0.11.0']

setup_kwargs = {
    'name': 'mines-data-engineering',
    'version': '0.1.4a4',
    'description': 'Helper package for the Data Engineering course at Colorado School of Mines',
    'long_description': '# Mines Data Engineering\n\nThis package simplifies the process of starting common database systems in the background on Singularity Hub.\n\n## Supported Databases\n\n- MongoDB\n- TimescaleDB (and Postgres)\n\n## Examples\n\n### MongoDB\n\n```python\nfrom mines_data_engineering import start_mongo\nimport pymongo\n\n# will need wait a couple minutes the first time this happens\n# while Singularity downloads and converts the Docker image\nconnection_string = start_mongo()\n\nclient = pymongo.MongoClient(connection_string)\nclient.my_db.my_col.insert_one({\'finally\': \'working\'})\n```\n\n### Postgres/TimescaleDB\n\n```python\nfrom mines_data_engineering import start_postgres\nimport psycopg2\n\n\n# will need wait a couple minutes the first time this happens\n# while Singularity downloads and converts the Docker image\nconnection_string = start_postgres()\n\nconn = psycopg2.connect(connection_string)\ncur = conn.cursor()\ncur.execute("SELECT 1")\nassert next(cur) == (1,)\n```\n',
    'author': 'Gabe Fierro',
    'author_email': 'gtfierro@mines.edu',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
